package com.cweb.controller;

import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.AESEncodeUtil;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.model.ResponseData;
import com.hai.service.HighDiscountAgentRelService;
import com.hai.service.HighDiscountCouponRelService;
import com.hai.service.HighDiscountUserRelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/4/4 14:46
 */
@Controller
@RequestMapping(value = "/discount")
@Api(value = "优惠券接口")
public class HighDiscountController {

    private static Logger log = LoggerFactory.getLogger(HighDiscountController.class);

    @Resource
    private HighDiscountAgentRelService highDiscountAgentRelService;

    @Resource
    private HighDiscountCouponRelService highDiscountCouponRelService;

    @Resource
    private HighDiscountUserRelService highDiscountUserRelService;

    @RequestMapping(value="/getDiscountByQrCode",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据二维码Code查询")
    public ResponseData getDiscountByQrCode(@RequestParam(name = "code", required = true) String code) {
        try {
            String relId;
            try {
                relId = AESEncodeUtil.aesDecrypt(code);
            } catch (Exception e) {
                log.error("HighDiscountController -> getDiscountByQrCode() error!","code解码错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "code解码错误");
            }
            return ResponseMsgUtil.success(highDiscountAgentRelService.getRelById(Long.parseLong(relId)));

        } catch (Exception e) {
            log.error("HighDiscountController -> getDiscountByQrCode() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getCouponByDiscount",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据优惠券查询卡券")
    public ResponseData getCouponByDiscount(@RequestParam(name = "discountId", required = true) Long discountId) {
        try {

            return ResponseMsgUtil.success(highDiscountCouponRelService.getRelByDiscount(discountId));

        } catch (Exception e) {
            log.error("HighDiscountController -> getCouponByDiscount() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

}
