package com.cweb.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.SessionObject;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.DateUtil;
import com.hai.common.utils.IDGenerator;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.entity.*;
import com.hai.model.HighCouponModel;
import com.hai.model.HighUserModel;
import com.hai.model.ResponseData;
import com.hai.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/3/26 23:08
 */
@Controller
@RequestMapping(value = "/highOrder")
@Api(value = "订单接口")
public class HighOrderController {

    private static Logger log = LoggerFactory.getLogger(HighMerchantStoreController.class);

    @Autowired
    private UserCenter userCenter;

    @Resource
    private HighOrderService highOrderService;

    @Resource
    private HighCouponService highCouponService;

    @Resource
    private HighCouponCodeService highCouponCodeService;

    @Resource
    private HighDiscountUserRelService highDiscountUserRelService;

    @Resource
    private HighDiscountCouponRelService highDiscountCouponRelService;

    @Resource
    private HighUserService highUserService;

    @RequestMapping(value="/addOrder",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "增加订单")
    public ResponseData addOrder(@RequestBody HighOrder highOrder, HttpServletRequest request) {
        try {

            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();

            if (highOrder.getHighChildOrderList() == null
                || highOrder.getHighChildOrderList().size() == 0
            ) {
                log.error("HighOrderController --> addOrder() error!", "参数错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            BigDecimal totalPrice = new BigDecimal("0");
            for (HighChildOrder childOrder : highOrder.getHighChildOrderList()) {
                if (childOrder.getGoodsType() == null || childOrder.getGoodsId() == null || childOrder.getSaleCount() == null) {
                    log.error("HighOrderController --> addOrder() error!", "参数错误");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
                }

                // 商品类型   1:卡卷  2:金币充值
                if (childOrder.getGoodsType() == 1) {
                    if (childOrder.getSaleCount() != 1) {
                        log.error("HighOrderController --> addOrder() error!", "卡卷只能购买一张");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "卡卷只能购买一张");
                    }
                    // 查询卡卷信息
                    HighCoupon coupon = highCouponService.getCouponById(childOrder.getGoodsId());
                    if (coupon == null) {
                        log.error("HighOrderController --> addOrder() error!", "未找到卡卷信息");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.NOT_FOUND_COUPON, "");
                    }

                    //  校验优惠券是否可用
                    if (highOrder.getMemDiscountId() != null) {
                        List<HighDiscountCouponRel> discountCouponRelList = highDiscountCouponRelService.getRelByCoupon(coupon.getId());
                        if (discountCouponRelList.size() > 0) {
                            List<HighDiscountCouponRel> collect = discountCouponRelList.stream().filter(o -> o.getDiscountId().equals(highOrder.getMemDiscountId())).collect(Collectors.toList());
                            if (collect == null || collect.size() == 0) {
                                log.error("HighOrderController --> addOrder() error!", "无法使用优惠券");
                                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法使用优惠券");
                            }
                        }
                    }


                    // 是否到底限购数量
                    if (highCouponService.userBuyLimitNumber(userInfoModel.getHighUser().getId(), coupon.getId()) == true) {
                        log.error("HighOrderController --> addOrder() error!", "已达到限购数量");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, coupon.getCouponName() + ",已达到限购数量");
                    }
                    if (highCouponCodeService.getStockCountByCoupon(coupon.getId()) <= 0) {
                        log.error("HighOrderController --> addOrder() error!", "卡卷库存数量不足");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COUPON_STOCK_INSUFFICIENT, "");
                    }
                    childOrder.setGoodsName(coupon.getCouponName());
                    childOrder.setGoodsImg(coupon.getCouponImg());
                    childOrder.setGoodsSpecName("默认");
                    childOrder.setGoodsPrice(coupon.getDiscountPrice());
                }

                if (childOrder.getGoodsType() == 2) {
                    if (childOrder.getGoodsPrice() == null) {
                        log.error("HighOrderController --> addOrder() error!", "请设置充值金额");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "请设置充值金额");
                    }
                    // 查询用户信息
                    HighUser user = highUserService.findByUserId(childOrder.getGoodsId());
                    if (user == null) {
                        log.error("HighOrderController --> addOrder() error!", "未找到用户");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.UN_MEMBER_ERROR, "");
                    }

                    childOrder.setGoodsName(user.getName());
                    childOrder.setGoodsImg(user.getHeaderImg());
                    childOrder.setGoodsSpecName("默认");
                }

                childOrder.setTotalPrice(childOrder.getGoodsPrice().multiply(new BigDecimal(childOrder.getSaleCount().toString())));
                childOrder.setGiveawayType(false);  // 是否是赠品  0：否   1：是
                childOrder.setChildOrdeStatus(1);   // 1 待支付  2 已支付  3.已完成  4. 已退款  5.已取消
                childOrder.setPraiseStatus(0);

                // 累计订单价格
                totalPrice = totalPrice.add(childOrder.getTotalPrice());
            }

            // 是否使用了优惠券
            if (highOrder.getMemDiscountId() != null) {
                // 卡优惠券信息
                HighDiscountUserRel rel = highDiscountUserRelService.getRelById(highOrder.getMemDiscountId());
                if (rel == null || rel.getStatus() != 1) {
                    log.error("HighOrderController --> addOrder() error!", "优惠券状态错误");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "优惠券状态错误");
                }
                highOrder.setMemDiscountName(rel.getHighDiscount().getDiscountName());
                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (rel.getHighDiscount().getDiscountType() == 1) {
                    // 如果订单总额 小于 满减价格
                    if (highOrder.getTotalPrice().compareTo(rel.getHighDiscount().getDiscountCondition()) > 1) {
                        log.error("HighOrderController --> addOrder() error!", "订单未达到满减额度");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.UN_MEMBER_ERROR, "订单未达到"+rel.getHighDiscount().getDiscountCondition()+"元,无法使用此优惠券");
                    }
                    // 订单总额 - 满减额度
                    BigDecimal total = highOrder.getTotalPrice().subtract(rel.getHighDiscount().getDiscountPrice());
                    // 如果总额小于0
                    if (total.compareTo(new BigDecimal("0")) == -1) {
                        highOrder.setTotalPrice(new BigDecimal("0"));
                    } else {
                        highOrder.setTotalPrice(total);
                    }
                }

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (rel.getHighDiscount().getDiscountType() == 2) {
                    // 订单总额 - 满减额度
                    BigDecimal total = highOrder.getTotalPrice().subtract(rel.getHighDiscount().getDiscountPrice());
                    // 如果总额小于0
                    if (total.compareTo(new BigDecimal("0")) == -1) {
                        highOrder.setTotalPrice(new BigDecimal("0"));
                    } else {
                        highOrder.setTotalPrice(total);
                    }
                }

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (rel.getHighDiscount().getDiscountType() == 3) {
                    // 折扣除100
                    BigDecimal discountPrice = rel.getHighDiscount().getDiscountPrice().divide(new BigDecimal("100"));
                    // 订单总额 * 折扣
                    BigDecimal total = highOrder.getTotalPrice().multiply(discountPrice);
                    highOrder.setTotalPrice(total);
                }
            }

            highOrder.setOrderNo("HF" + DateUtil.date2String(new Date(),"yyyyMMddHHmmss") + IDGenerator.nextId(5));
            highOrder.setMemId(userInfoModel.getHighUser().getId());
            highOrder.setMemName(userInfoModel.getHighUser().getName());
            highOrder.setMemPhone(userInfoModel.getHighUser().getPhone());
            highOrder.setPayPrice(totalPrice);
            highOrder.setCreateTime(new Date());
            highOrder.setOrderStatus(1);

            highOrderService.insertOrder(highOrder);

            return ResponseMsgUtil.success(highOrder);

        } catch (Exception e) {
            log.error("HighOrderController -> addOrder() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getOrderById", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据id查询订单详情")
    public ResponseData getOrderById(@RequestParam(name = "orderId", required = true) Long orderId) {
        try {

            return ResponseMsgUtil.success(highOrderService.getOrderById(orderId));

        } catch (Exception e) {
            log.error("HighOrderController --> getOrderById() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

/*
    @RequestMapping(value = "/calculateOrder", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "计算订单详情")
    public ResponseData calculateOrder(@RequestParam(name = "orderId", required = true) Long orderId,
                                       @RequestParam(name = "userDiscountId", required = false) Long userDiscountId) {
        try {

            HighOrder order = highOrderService.getOrderById(orderId);
            if (order != null && userDiscountId != null) {
                // 卡优惠券信息
                HighDiscountUserRel rel = highDiscountUserRelService.getRelById(userDiscountId);
                order.setMemDiscountId(userDiscountId);
                order.setMemDiscountName(rel.getHighDiscount().getDiscountName());

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (rel.getHighDiscount().getDiscountType() == 1) {
                    // 如果订单总额 小于 满减价格
                    if (order.getTotalPrice().compareTo(rel.getHighDiscount().getDiscountCondition()) > 1) {
                        log.error("HighOrderController --> addOrder() error!", "订单未达到满减额度");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.UN_MEMBER_ERROR, "订单未达到"+rel.getHighDiscount().getDiscountCondition()+"元,无法使用此优惠券");
                    }
                    // 订单总额 - 满减额度
                    BigDecimal totalPrice = order.getTotalPrice().subtract(rel.getHighDiscount().getDiscountPrice());
                    // 如果总额小于0
                    if (totalPrice.compareTo(new BigDecimal("0")) == -1) {
                        order.setTotalPrice(new BigDecimal("0"));
                    } else {
                        order.setTotalPrice(totalPrice);
                    }
                }

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (rel.getHighDiscount().getDiscountType() == 2) {
                    // 订单总额 - 满减额度
                    BigDecimal totalPrice = order.getTotalPrice().subtract(rel.getHighDiscount().getDiscountPrice());
                    // 如果总额小于0
                    if (totalPrice.compareTo(new BigDecimal("0")) == -1) {
                        order.setTotalPrice(new BigDecimal("0"));
                    } else {
                        order.setTotalPrice(totalPrice);
                    }
                }

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (rel.getHighDiscount().getDiscountType() == 3) {
                    // 折扣除100
                    BigDecimal discountPrice = rel.getHighDiscount().getDiscountPrice().divide(new BigDecimal("100"));
                    // 订单总额 * 折扣
                    BigDecimal totalPrice = order.getTotalPrice().multiply(discountPrice);
                    order.setTotalPrice(totalPrice);
                }
            }
            return ResponseMsgUtil.success(order);

        } catch (Exception e) {
            log.error("HighOrderController --> cancelOrder() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }
*/

    @RequestMapping(value = "/cancelOrder", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "取消订单")
    public ResponseData cancelOrder(@RequestParam(name = "orderId", required = true) Long orderId) {
        try {

            highOrderService.cancelOrder(orderId);

            return ResponseMsgUtil.success("操作成功");

        } catch (Exception e) {
            log.error("HighOrderController --> cancelOrder() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getUserOrderList", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "获取用户订单")
    public ResponseData getUserOrderList(
            @RequestParam(name = "status", required = false) Integer status,
            @RequestParam(name = "orderNo", required = false) String orderNo,
            @RequestParam(name = "pageNum", required = true) Integer pageNum,
            @RequestParam(name = "pageSize", required = true) Integer pageSize,
                                         HttpServletRequest request) {
        try {

            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();

            Map<String,Object> map = new HashMap<>();
            map.put("memId", userInfoModel.getHighUser().getId());
            map.put("status", status);
            map.put("orderNo", orderNo);

            PageHelper.startPage(pageNum,pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(highOrderService.getOrderList(map)));

        } catch (Exception e) {
            log.error("HighOrderController --> getUserOrderList() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }
}
